package domain_test

import (
	"net/url"
	"testing"

	"gitea.dwysokinski.me/twhelp/corev3/internal/domain"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestNewSyncServersCmdPayload(t *testing.T) {
	t.Parallel()

	type args struct {
		versionCode string
		url         *url.URL
	}

	tests := []struct {
		name        string
		args        args
		expectedErr error
	}{
		{
			name: "OK",
			args: args{
				versionCode: "pl",
				url: &url.URL{
					Scheme: "https",
					Host:   "plemiona.pl",
				},
			},
		},
	}

	for _, tt := range tests {
		tt := tt

		t.Run(tt.name, func(t *testing.T) {
			t.Parallel()

			payload, err := domain.NewSyncServersCmdPayload(tt.args.versionCode, tt.args.url)
			require.ErrorIs(t, err, tt.expectedErr)
			if tt.expectedErr != nil {
				return
			}
			assert.Equal(t, tt.args.versionCode, payload.VersionCode())
			assert.Equal(t, tt.args.url, payload.URL())
		})
	}
}

func TestNewSyncServersCmdPayloadWithStringURL(t *testing.T) {
	t.Parallel()

	type args struct {
		versionCode string
		url         string
	}

	tests := []struct {
		name        string
		args        args
		expectedErr error
	}{
		{
			name: "OK",
			args: args{
				versionCode: "pl",
				url:         "https://plemiona.pl",
			},
		},
		{
			name: "ERR: invalid url",
			args: args{
				versionCode: "pl",
				url:         "plemiona.pl",
			},
			expectedErr: domain.InvalidURLError{
				URL: "plemiona.pl",
			},
		},
	}

	for _, tt := range tests {
		tt := tt

		t.Run(tt.name, func(t *testing.T) {
			t.Parallel()

			payload, err := domain.NewSyncServersCmdPayloadWithStringURL(tt.args.versionCode, tt.args.url)
			require.ErrorIs(t, err, tt.expectedErr)
			if tt.expectedErr != nil {
				return
			}
			assert.Equal(t, tt.args.versionCode, payload.VersionCode())
			assert.Equal(t, tt.args.url, payload.URL().String())
		})
	}
}

type serverKeyValidationTest struct {
	name        string
	key         string
	expectedErr error
}

func newServerKeyValidationTests() []serverKeyValidationTest {
	return []serverKeyValidationTest{
		{
			name: "ERR: server key length < 1",
			expectedErr: domain.LenError{
				Min: 1,
				Max: 10,
			},
		},
		{
			name: "ERR: server key length > 10",
			key:  "keykeykeyke",
			expectedErr: domain.LenError{
				Min:     1,
				Max:     10,
				Current: len("keykeykeyke"),
			},
		},
	}
}
